function xF = find_periodic_md(fhandle, period, range, init_num, tolerance, diverge, max_step, rnd, varargin)

%
% Finding periodic points of the specified period, probing
% the defined number of initial conditions, which are either
% set randomly or distributed regularly in the given
% interval (multi-dimensional)
% 
% fhandle   - function handle 
% period    - period of points to search
% range     - range for searching perioodic points 
% init_num  - the number of initial conditions to try
% tolerance - checking point accuracy
% diverge - limit for divergence to infinity
% max_step  - maximal number of Newton steps exceeding this
%             value means that the Newton method do not
%             converge 
% rnd       - set initial conditions randomly or not
%             if true (or 1), then initial conditions are
%             chosen randomly (in the total amount of
%             init_num) if false (or 0), then the search
%             interval is divided into init_num equal
%             subdomains, and a middle point of each
%             subdomain is taken for the initial condition
% varargin  - cell array of the map parameters
%
% return values:
% xF - the vector of periodic points found sorted in
%      ascending order on the first coordinate
%


xF = [];

% checking if there exists an explicit Jacobi matrix function
fh = functions(fhandle);
Jname = ['@' fh.function '_J'];
Jhandle = str2func(Jname);
Jh = functions(Jhandle);
if strcmp(Jh.file, '')   % there is no file with Jacobi matrix function
    cprintf(get_colour('warn'), 'WARNING: The function''s Jacobi matrix is not defined\n');
    cprintf(get_colour('warn'), '         and will be calculated approximately.\n\n');
    has_diff = 0;
else
    has_diff = 1;
end

if (init_num == 0)
    cprintf(get_colour('vital'), 'Using the given initial condition(s).\n\n');
    sz = size(range);
    dim = sz(2); % number of columns defines the dimension
    total = sz(1); % number of rows is the number of conditions to try
else
    sz = size(range);
    if (sz(2) ~= 2)
        cprintf('err', 'ERROR: interval (arg #3) must be a vector with two columns.\n');
        return;
    end
    
    % dimension now is defined by range
    % if it does not correspond the function dimension, 
    % there will be an error later
    dim = sz(1);
    
    if (~rnd)
        % if requiring the subdivision of range
        % the length of init_num must be the same as the map dimension 
        % adding ones for the missing elements if needed
        if (length(init_num) < dim)
            for i = (length(init_num) + 1):dim
                init_num(i) = 1;
            end
        end
        
        total = 1;
        for i = 1:dim
            total = total*init_num(i);
        end
        
        % make an appropriate subdivision of range
        div = make_subdiv(range, init_num);
    else
        total = init_num;
    end;
end

I = eye(dim, dim);

% applying Newton method for each initial condition
x_all = [];
x0 = [];
x_init = [];
for j = 1:total
    
    cprintf(get_colour('aux'), 'Run number %d\n', j);
        
    if (init_num == 0)
        x0 = range(j, :);
    else
        if (rnd) % choose randomly
            for i = 1:dim
                x0(i) = range(i, 1) + (range(i, 2) - range(i, 1)).*rand;
            end
        else % take x0 as a middle point of a subdomain
            for i = 1:dim
                x0(i) = (div{j}(i, 1) + div{j}(i, 2))/2;
            end 
        end
    end
                
    x_init = x0;
    
    k = 1;
    while (k <= max_step)
        % iterate the function period times to get the final Jacobian
        f0 = x0;
        J = eye(dim, dim);
        for i = 1:period
            if (has_diff)
                J = Jhandle(f0, varargin{:})*J;
            else
                J = jacob(fhandle, f0, 0.0001, varargin{:})*J;
            end
            f0 = fhandle(f0, varargin{:});
        end
        
        while (abs(det(J - I)) < 1e-6)
            % if the matrix (J - I) is singular
            % (i. e. the function is close to the linear)
            % we add a displacement in the direction orthogoal 
            % to the vector corresponding to 'zero'
            % eigenvalue (ref ???)
            cprintf(get_colour('warn'), 'The matrix (J-I) is singular');
            [V, D] = eig(J - I);  % find eigenvalues and -vectors
            noZeros = find(D);
            %%% There will be a problem here if (J - I) = 0
            %%% see Numerical Recepies, Sec. 9.7
            [small, small_id] = min(abs(D(noZeros)));
            x0(1) = x0(1) - V(2, small_id);
            x0(2) = x0(2) + V(1, small_id);

            % find dx with new initial condition
            f0 = x0;
            J = eye(dim, dim);
            for i = 1:period
                if (has_diff)
                    J = Jhandle(f0, varargin{:})*J;
                else
                    J = jacob(fhandle, f0, 0.0001, varargin{:})*J;
                end
                f0 = fhandle(f0, varargin{:});
            end
        end
        % the matrix (J - I) is not degenerate
        % continuing usual procedure
        dx = - ((J - I)\(f0 - x0)')';
        
        
        % new value of solution approximation
        x1 = x0 + dx;
        % period'th iterate of function for new approximation
        %f1 = x1;
        %for i = 1:period
        %    f1 = fhandle(f1, varargin{:});
        %end
        [f1, stat] = iterate(fhandle, x1, period, diverge, varargin{:});
        if(stat == -1)
            cprintf(get_colour('warn'), 'WARNING: during map iteration orbit diverges, stop.\n');
            break;
        end;
        
        % checking if the new approximation also
        % minimizes the scalar product 1/2*f*f
        % if not, adjusting step
        lambda0 = 1;
        lambda1 = 0;
        %lambda2 = 0; % temporal
        g0 = 1/2*dot((f0 - x0), (f0 - x0));
        gl = 1/2*dot((f1 - x1), (f1 - x1));
        gl0 = gl;
        while (gl > g0)   % approximation is not appropriate
            if (lambda1 == 0)  % first adjustment, quadratic
                lambda1 = max([0.1 g0/(gl + g0)]);
                
                % new approximation
                x1 = x0 + lambda1*dx;
                f1 = x1;
                for i = 1:period
                    f1 = fhandle(f1, varargin{:});
                end
                gl = 1/2*dot((f1 - x1), (f1 - x1));
                gl1 = gl;
            else  % other adjsutments, cubic
                a = 1/(lambda1 - lambda0)*dot([1/lambda1^2 -1/lambda0^2], ...
                    [gl1 + (2*lambda1 - 1)*g0; gl0 + (2*lambda0 - 1)*g0]);
                b = 1/(lambda1 - lambda0)*dot([-lambda0/lambda1^2 lambda1/lambda0^2], ...
                    [gl1 + (2*lambda1 - 1)*g0; gl0 + (2*lambda0 - 1)*g0]);
                lambda2 = max([(- b + sqrt(b^2 + 6*a*g0))/(3*a), 0.1*lambda1]);
                lambda2 = min([lambda2, 0.5*lambda1]);

                % new approximation
                x1 = x0 + lambda2*dx;
                f1 = x1;
                for i = 1:period
                    f1 = fhandle(f1, varargin{:});
                end
                gl = 1/2*dot((f1 - x1), (f1 - x1));
                gl0 = gl1;
                gl1 = gl;

                lambda0 = lambda1;
                lambda1 = lambda2;
            end
            
        end
    
        % stop if error is small enough
        if norm(f1 - x1) < tolerance
            x_all = [x_all; x1];
            cprintf(get_colour('vital'), 'Total number of Newton steps is %d.\n\n', k);
            break;
        end
        
        x0 = x1;
        k = k + 1;
    end
    
    if (k > max_step)
        cprintf(get_colour('warn'), 'WARNING: The Newton method failed to converge.\n');
        cprintf(get_colour('warn'), '         The solution has not been found.\n');
        cprintf(get_colour('warn'), '         Initial condition was %.8f', x_init(1));
        for i = 2:dim
            cprintf(get_colour('warn'), ', %.8f', x_init(i));
        end
        cprintf(get_colour('warn'), '\n\n');
    end

end

% eliminating repeated values

cprintf(get_colour('final'), 'The total number of initial conditions checked is %d.\n', total);
if (isempty(x_all))
    cprintf([0.7, 0, 0], 'No points were found.\n');
else
    % sorting found solutions in increasing order
    [xF, idx] = sort(x_all(:, 1));
    x_all = x_all(idx, :);

    % saving only values being distinct enough
    %%% If there are two distinct solutions which are very
    %%% close, this may cause inaccuracy, as they are
    %%% treated equal 
    m = length(x_all(:, 1));
    xF = [x_all(1, :)];
    for i = 2:m
        % if the adjusent solutions are not close
        % they should be different periodic points
        if (norm(x_all(i, :) - x_all(i-1, :)) > tolerance*10)
            xF = [xF; x_all(i, :)];
        end
    end
    
    cprintf(get_colour('final'), 'The total number of distinct points found is %d.\n', ...
            length(xF(:, 1)));
end



