function [param1, param2, periods] = ...
        period_bd_calc(fhandle, x0, prange, dp, pidx, ...
        trans, maxperiod, tolerance, diverge, varargin)

%
% Calculate the data for the 2D period diagram
% 
% fhandle    - function handle 
% x0         - initial point
% prange     - range for the 2 parameters
%              2x2 matrix with the 1st row for param1, 2nd
%              row for param2 
% dp         - 2-dim vector of parameter steps
% pidx       - 2-dim vector of parameter indices
% trans      - transient
% maxperiod  - maximal period to check
% tolerance  - checking point accuracy
% diverge - limit for divergence to infinity
% varargin   - cell array of the map parameters
%
% return values:
% param1  - multi-dim vector with the first parameter mesh
% param2  - multi-dim vector with the second parameter mesh
% periods - 2-dim matrix with the periods related to the
%           parameter mesh (-1 means diverge, 0 means
%           higher periods or chaos)
%



param1 = [];
param2 = [];


% checking whether initial condition is given 
% or needed to be get randomly
if (isnumeric(x0))
    x0dim = 0;
else % x0 is not a numeric array
    x0cell = regexp(x0, 'rand\([1-9][0-9]*\)', 'match');
    if (length(x0cell) ~= 1) 
        cprintf('err', 'ERROR: Illegal inital condition, refer to the user guide.\n');
        periods = [];
        return;
    else
        x0dim = regexp(x0cell, '[1-9][0-9]*', 'match');
        if (length(x0dim) ~= 1)
            cprintf('err', 'ERROR: Illegal inital condition, refer to the user guide.\n');
            periods = [];
            return;
        else
            x0dim = cell2mat(x0dim{1});
            x0dim = str2num(x0dim);
        end
    end
end


% checking parameter range
sz = size(prange);
if ((sz(1) ~= 2) || (sz(2) ~= 2))
    cprintf('err', 'ERROR: Illegal range, prange (arg #3) must be a 2x2 matrix.\n');
    periods = [];
    return;
end;

% checking parameter step
sz = length(dp);
if (sz ~= 2)
    cprintf('err', 'ERROR: Illegal parameter step, dp (arg #4) must be a 2-dim vector.\n');
    periods = [];
    return;
end;

% checking parameter indices
sz = length(pidx);
if (sz ~= 2)
    cprintf('err', 'ERROR: Illegal indices, pidx (arg #5) must be a 2-dim vector.\n');
    periods = [];
    return;
end;



varargin{pidx(1)} = prange(1, 1);
p1 = 1;
nRun = 0;
nRunTotal = ceil((prange(1, 2) - prange(1, 1))/dp(1));

while (varargin{pidx(1)} <= prange(1, 2))
    % loop on the first parameter
    cprintf(get_colour('aux'), 'First parameter = %f\n', varargin{pidx(1)});
    
    tStart = tic;

    if (nRun >= 1)
        tLeft = round(tDelta*(nRunTotal - nRun)); % total time in seconds
        tLeftSec = 0;
        tLeftMin = 0;
        tLeftHour = 0;
        
        if (tLeft > 60) % more than a minute
            tLeftSec = mod(tLeft, 60);
            tLeft = (tLeft - tLeftSec)/60; % total time in minutes
            if (tLeft > 60) % more than an hour
                tLeftMin = mod(tLeft, 60);
                tLeftHour = (tLeft - tLeftMin)/60; % time in hours
            else % less than an hour
                tLeftMin = tLeft;
                tLeftHour = 0;
            end
        else % less than a minute
            tLeftSec = tLeft;
            tLeftMin = 0;
        end
    
        if (tLeftHour ~= 0)
            cprintf(get_colour('vital'), ...
                    'Approximate time left is %d hours %d minutes %d seconds\n\n', ...
                    tLeftHour, tLeftMin, tLeftSec);
        elseif (tLeftMin ~= 0)
            cprintf(get_colour('vital'), ...
                    'Approximate time left is %d minutes %d seconds\n\n', ...
                    tLeftMin, tLeftSec);
        else
            cprintf(get_colour('vital'), 'Approximate time left is %d seconds\n\n', ...
                    tLeftSec);
        end
    end
    
    nRun = nRun + 1;
    

    param1 = [param1; varargin{pidx(1)}];
    varargin{pidx(2)} = prange(2, 1);
    p2 = 1;
    
    while (varargin{pidx(2)} <= prange(2, 2))
        
        % loop on the second parameter
        
        if (p1 == 1)
            param2 = [param2; varargin{pidx(2)}];
        end;
        
        if (x0dim == 0) % initial condition is given explicitely
            x = x0;
        else % random initial condition
            x = rand(1, x0dim);
        end
        
        x = iterate(fhandle, x, trans, diverge, varargin{:});
        
        if (norm(x) > diverge)
            % trajectory seems to diverge to infinity
            periods(p2, p1) = -1;
        else
            x1 = x;
            for i = 1:(maxperiod+1)
                x1 = fhandle(x1, varargin{:});
                if (norm(x1 - x) < tolerance)
                    break;
                end;
            end;
            
            if (i > maxperiod)
                % point is of larger period, or chaotic
                periods(p2, p1) = 0;
            else
                % period of point is the last i-value
                periods(p2, p1) = i;
            end;
        end;
            
        varargin{pidx(2)} = varargin{pidx(2)} + dp(2);
        p2 = p2 + 1;
    end;

    varargin{pidx(1)} = varargin{pidx(1)} + dp(1);
    p1 = p1 + 1;
    
    tDelta = toc(tStart);
    if (nRun == 1)
        cprintf(get_colour('vital'), 'Time elapsed for one run is about %.2f seconds\n\n', tDelta);
    end;
end;






