function [p_half, f_half, stat] = bisection(fhandle, iters, p_cur, p_start, p_end, ...
    Dk, eB, diverge, varargin)

% 
% Bisection algorithm to find the point in the interval
% [p_start, p_end] which is mapped by fhandle near the point
% p_cur at a distance (1 - eB)*Dk < dist < (1 + eB)*Dk 
%
% fhandle        - function (dynamical system) handle
% iters          - number of iterates needed
% p_cur          - the last computed manifold point
% p_start, p_end - ends of segment, containing the searched preimage 
% Dk             - current step size estimate
% eB             - bisection tolerance
% varargin       - cell array of the function parameters
%
%
% return values:
% p_half - point estimate, lies in [p_start, p_end]
% f_half - point estimate image (lies at distance about Dk from p_cur)
% stat   - exit status (0: the estimate was found, 1:
%          tolerance failed (the last computed value is
%          returned)
%

% maximal bisection steps to be performed 
load('data/default.mat', 'bstep_max');

stat = 0;


p_half = 0.5*p_start + 0.5*p_end;
f_half = iterate(fhandle, p_half, iters, diverge, varargin{:});


bstep = 0;
while ((abs(norm(f_half - p_cur) - Dk) > eB) && (bstep <= bstep_max))
    % checking whether f_half lies inside the circle 
    if (norm(f_half - p_cur) < Dk)
        % f_half lies inside the circle
        p_start = p_half;
    else
        % f_half lies outside the circle
        p_end = p_half;
    end;
        
    p_half = 0.5*p_start + 0.5*p_end;
    f_half = iterate(fhandle, p_half, iters, diverge, varargin{:});

    bstep = bstep + 1;
end
    
if (bstep > bstep_max)
    % the tolerance still not reached
    % returning the last computed values of p_half and f_half
    stat = 1;
    cprintf(get_colour('warn'), ['WARNING: After %d iterations bisection ' ...
            'error is still large.\n'], max_iter);
    cprintf(get_colour('warn'),  '         Solution may be inaccurate');
end

    
    
